!
! quadrature --- module to numerically integrate a function on a grid
! Christian Buth
! 22 Aug 2006
!
! Version 1.0.0: 22 Aug 2006 (Newton Cotes formulas: midpoint rule,
!       trapezoid rule and Simpson's rule implemented)
! Version 1.1.0: 29 Aug 2006 (3/8-rule)
! Version 1.2.0:  1 Sep 2006 (Loads of inaccuracies removed)
!
! Gene Howard Golub and James M. Ortega, Scientific computing and
! differential equations: an introduction to numerical methods,
! Academic Press, San Diego, New York, Boston, London, Sydney,
! Tokyo, Toronto, 1992, ISBN 0-12-289255-0
!
! Josef Stoer, Numerische Mathematik 1, 7. Auflage, Springer Berlin,
! Heidelberg, New York, London, Paris, Tokyo, Hong Kong, Barcelona,
! Budapest, 1994, ISBN 3-540-57823-4
!


      module quadrature
      implicit none

      contains

!
! midpointrule --- apply the composite midpoint rule to an array
!       of points.  Note that the first and last point of the
!       array are not used
!

      real(dblprec) function midpointrule(ngrid, xh, func)
      use fortrantl
      implicit none

      integer, intent(in) :: ngrid
      real(dblprec), intent(in) :: xh, func(ngrid)
      integer i
      real(dblprec) r

      r = 0.0D0
      do i=2, ngrid-1
        r = r + func(i)
      end do

      midpointrule = xh * r
      end function midpointrule


!
! trapezoidrule --- apply the composite trapezoid rule to an array
!       of points
!

      real(dblprec) function trapezoidrule(ngrid, xh, func)
      use fortrantl
      implicit none

      integer, intent(in) :: ngrid
      real(dblprec), intent(in) :: xh, func(ngrid)
      integer i
      real(dblprec) r

      r = 0.5D0 * (func(1) + func(ngrid))
      do i=2, ngrid-1
        r = r + func(i)
      end do

      trapezoidrule = xh * r
      end function trapezoidrule


!
! simpsonsrule --- apply the composite Simpson's rule to an
!       array of points with an odd number of entries
!

      real(dblprec) function simpsonsrule(ngrid, xh, func)
      use fortrantl
      implicit none

      integer, intent(in) :: ngrid
      real(dblprec), intent(in) :: xh, func(ngrid)
      integer i
      real(dblprec) r

      if (.not. btest(ngrid, 0)) then
        stop 'simpsonsrule: the number of grid points must be odd!'
      end if

      r = func(1) - func(ngrid)
      do i=2, ngrid-1, 2
        r = r + 4.0D0 * func(i) + 2.0D0 * func(i+1)
      end do

!       ! Sum interval by interval
!       r = 0.0D0
!       do i=1, ngrid-2, 2
!         r = r + func(i) + 4.0D0 * func(i+1) + func(i+2)
!       end do

      simpsonsrule = xh / 3.0D0 * r
      end function simpsonsrule


!
! threeeighthsrule --- apply the composite 3/8-rule to an
!       array of points with an number of entries which is
!       a multiple of four
!

      real(dblprec) function threeeighthsrule(ngrid, xh, func)
      use fortrantl
      implicit none

      integer, intent(in) :: ngrid
      real(dblprec), intent(in) :: xh, func(ngrid)
      integer i
      real(dblprec) r

      if (mod(ngrid-1, 3) == 1) then
        stop 'threeeighthsrule: the number of grid points minus one must be a multiple of three!'
      end if

      r = 0.0D0
      do i=1, ngrid-3, 3
        r = r + func(i) + 3.0D0 * func(i+1) + 3.0D0 * func(i+2) + func(i+3)
      end do

      threeeighthsrule = 3.0D0 * xh / 8.0D0 * r
      end function threeeighthsrule
      end module quadrature
