!
! SESWmtrx --- Calculate matrix representation of the difference of the
!       atomic central potential, i.e. the Coulomb potential plus the
!       centrifugal potential, and the smooth exterior complex
!       scaled (SES) one
! Christian Buth
! 24 May 2006
!
!
! in eigenfunction
! representation, r being the radial coordinate.  The integrals are evaluated
! upon the substitution x=sqrt(r) (r is distributed on a quadratic mesh,
! x on a linear one). xh is the spacing of the linear mesh.
!
!
! ngrd  : number of grid points of the quadratic mesh
! xh    : spacing of points in the linear x=sqrt(r)-mesh
! egfxn : eigenfunctions of the atomic Hamiltonian with the
!         Herman-Skillman potential represented
!         in the finite element basis
!


      subroutine SESpotmtrx(potmat, nstate, nl, egfxn, ngrd, xh, lastpotentialvalue, ipottype)
      use SESpath
      use quadrature
      implicit none

      integer nstate, nl, ngrd, ipottype
      integer i, j, k, l
      real(dblprec) egfxn(ngrd,nstate,nl)
      real(dblprec), allocatable :: rework(:), imwork(:)
      real(dblprec) r, x, xh, numinterr, p, re1, im1, re2, im2, lastpotentialvalue
      complex(dblprec) potmat(nstate,nstate,nl)
      complex(dblprec) value, z

      allocate(rework(ngrd), imwork(ngrd))
      numinterr = 0.0D0

      ! Sample the SES atomic central potential minus the unscaled
      ! atomic potential on the numeric (quadratic) integration grid
      do l = 1, nl
        do k = 1, nstate
          do j = 1, k
            do i = 1, ngrd
              x = xh * real(i, dblprec)
              rework(i) = egfxn(i,j,l) * egfxn(i,k,l)           ! Eigenfunction values
              rework(i) = 2.0D0 * x * rework(i)                 ! Integration measure
              imwork(i) = rework(i)

              ! Atomic potential
!               p = lastpotentialvalue / x**2
!               z = cmplx(lastpotentialvalue, 0.0D0, dblprec) / pathFc(x**2)
              ! Bain
!               value = 0.0D0
!               p = 7.5D0 * x**4 * exp(-x**2)
!               z = (7.5D0, 0.0D0) * pathFc(x**2)**2 * exp(-pathFc(x**2))
              ! Imke
!               p = -4.0D0 / x**2
!               z = (-4.0D0,0.0D0) / pathFc(x**2)

              select case (ipottype)
                ! Spline interpolated potential
                case (0)
                  ! Centrifugal potential
                  p = real (l * (l + 1), dblprec) / (2.0D0 * x**4)
                  z = cmplx(l * (l + 1), 0.0D0, dblprec) / ((2.0D0,0.0D0) * pathFc(x**2)**2)
                  ! Conventional complex scaling from the origin, no CAP
                  if (originCS) then
                    value = cmplx(0.0D0, p, dblprec)
                  ! Smooth exterior complex scaling
                  else  
                    value = z - cmplx(p, 0.0D0, dblprec)
                  end if
                  
                  ! Atomic potential
                  p = lastpotentialvalue / x**2
                  z = cmplx(lastpotentialvalue, 0.0D0, dblprec) / pathFc(x**2)
                  ! Conventional complex scaling from the origin, no CAP
                  if (originCS) then
                    value = value + cmplx(p, 0.0D0, dblprec)
                  ! Smooth exterior complex scaling
                  else  
                    value = value + z - cmplx(p, 0.0D0, dblprec)
                  end if
                  
                ! Potential of Bain et al. from J. Phys. B *26* 4503 (1993)
                case (1)
                  p = 7.5D0 * x**4 * exp(-x**2)
                  z = (7.5D0, 0.0D0) * pathFc(x**2)**2 * exp(-pathFc(x**2))
                  value = z - cmplx(p, 0.0D0, dblprec)
                  
                ! Imke Britta Müller's potential from Int. J. Quantum
                ! Chem. *94* 75--92 (2003) where the l=1
                ! centrifugal potential term was replaced by the general
                ! term with l = ll.  Imke's results are obtained for
                ! calculations which include l=1.
                case (2)
                  p = real (l * (l + 1), dblprec) / (2.0D0 * x**4) - 4.0D0 / x**2  &
     &                  + 7.0D0 * exp(-6.0D0 * (x**2 - 6.0D0)**2)
                  z = cmplx(l * (l + 1), 0.0D0, dblprec) / ((2.0D0,0.0D0) * pathFc(x**2)**2)  &
     &                  + (-4.0D0,0.0D0) / pathFc(x**2)  &
     &                  + (7.0D0, 0.0D0) * exp((-6.0D0, 0.0D0) * (pathFc(x**2) - (6.0D0, 0.0D0))**2)
                  value = z - cmplx(p, 0.0D0, dblprec)
                case default
                  stop 'SESpotmtrx: unknown potential requested!'
              end select

              rework(i) = rework(i) * real (value, dblprec)
              imwork(i) = imwork(i) * aimag(value)
            end do

            ! Numerical integration
            re1 = threeeighthsrule(ngrd,xh,rework)
            im1 = threeeighthsrule(ngrd,xh,imwork)
            re2 = simpsonsrule    (ngrd,xh,rework)
            im2 = simpsonsrule    (ngrd,xh,imwork)
            r = (re1 - re2)**2 + (im1 - im2)**2
            if (r > numinterr) then
              numinterr = r
            end if
            
            ! Conventional complex scaling from the origin, no CAP
            if (originCS) then
              potmat(j,k,l) = conjg(eit) * cmplx(re2, 0.0D0, dblprec)
              potmat(j,k,l) = potmat(j,k,l) + conjg(eit)**2 * cmplx(im2, 0.0D0, dblprec)
            ! Smooth exterior complex scaling
            else  
              potmat(j,k,l) = cmplx(re2, im2, dblprec)
            end if

            potmat(k,j,l) = potmat(j,k,l)          ! potmat is complex symmetric
          end do
        end do
      end do

      write (*,*) '        Maximum error in numerical integration ', sqrt(numinterr)
      deallocate(rework, imwork)
      end subroutine SESpotmtrx
