!
! fTfmtrx --- Calculate matrix representation of the transformed
!       kinetic energy operator f^-1 T f^-1 for the smooth
!       exterior complex scaled Hamiltonian
! Christian Buth
! 22 May 2006
!


! #define fltprec quadprec
#define fltprec dblprec


      subroutine fTfmtrx(fTf, nstate, nl, egfxn, egder, ngrd, xh)
      use fortrantl
      implicit none

      integer, intent(in) :: nstate, nl, ngrd
      real(dblprec), intent(in) :: xh
      real(dblprec), intent(in) :: egfxn(ngrd,nstate,nl), egder(ngrd,nstate,nl)
      complex(dblprec), intent(out) :: fTf(nstate,nstate,nl)

      call anader_fTfmtrx(fTf, nstate, nl, egfxn, egder, ngrd, xh)
      end subroutine fTfmtrx


!
! anader_fTfmtrx --- evaluate the transformed kinetic energy
!       operator for the SES CAP using analytical derivatives
!

      subroutine anader_fTfmtrx(fTf, nstate, nl, egfxn, egder, ngrd, xh)
      use quadrature
      use SESpath
      implicit none

      integer, intent(in) :: nstate, nl, ngrd
      real(dblprec), intent(in) :: xh
      real(dblprec), intent(in) :: egfxn(ngrd,nstate,nl), egder(ngrd,nstate,nl)
      complex(dblprec), intent(out) :: fTf(nstate,nstate,nl)
      integer i, j, k, l
      real(dblprec) pf
      real(dblprec) x, r, numinterr, suminterr, re1, im1, re2, im2
      real(dblprec), allocatable :: rework(:), imwork(:)
      complex(dblprec) z, zt

      numinterr = 0.0D0
      suminterr = 0.0D0
      allocate(rework(ngrd), imwork(ngrd))

      do l = 1, nl
         do k = 1, nstate
            do j = 1, k
               do i = 1, ngrd
                 x = xh * real(i, dblprec)
                 pf = -2.0D0 * x / 2.0D0      ! Integration
                                ! measure -hbar^2 / (2 m) in a.u. substitution
                                ! r = x^2 in the two derivatives

                 ! Unscaled kinetic energy
                 z =     cmplx(egder(i,j,l), kind=dblprec)
                 z = z * cmplx(egder(i,k,l), kind=dblprec)
                 zt = z
                   
                 ! Smooth exterior complex scaling
                 if (.not. originCS) then
                   ! First term
                   z =   - cmplx(egder(i,j,l), kind=dblprec)
                   z = z * cmplx(egder(i,k,l), kind=dblprec)
                   z = z / pathf(x**2)**2
                   zt = zt + z
                   ! Second term
                   z =     cmplx(egder(i,j,l), kind=dblprec)
                   z = z * cmplx(egfxn(i,k,l), kind=dblprec)
                   z = z / pathf(x**2)**3 * pathfp(x**2)
                   zt = zt + z
                   ! Third term
                   z =     cmplx(egfxn(i,j,l), kind=dblprec)
                   z = z * cmplx(egder(i,k,l), kind=dblprec)
                   z = z / pathf(x**2)**3 * pathfp(x**2)
                   zt = zt + z
                   ! Forth term
                   z =   - cmplx(egfxn(i,j,l), kind=dblprec)
                   z = z * cmplx(egfxn(i,k,l), kind=dblprec)
                   z = z / pathf(x**2)**4 * pathfp(x**2)**2
                   zt = zt + z
                 end if

                 zt = zt * cmplx(pf, kind=dblprec)
                 rework(i) = real(zt       , dblprec)
                 imwork(i) = real(aimag(zt), dblprec)
               end do

               ! Numerical integration
               re1 = threeeighthsrule(ngrd,xh,rework)
               re2 = simpsonsrule    (ngrd,xh,rework)
               ! Conventional complex scaling from the origin, no CAP
               if (originCS) then
                 im1 = 0.0D0
                 im2 = 0.0D0
               ! Smooth exterior complex scaling
               else  
                 im1 = threeeighthsrule(ngrd,xh,imwork)
                 im2 = simpsonsrule    (ngrd,xh,imwork)
               end if
               r = (re1 - re2)**2 + (im1 - im2)**2
               if (r > numinterr) then
                 numinterr = r
               end if
               suminterr = suminterr + sqrt(r)
               
               ! Conventional complex scaling from the origin, no CAP
               if (originCS) then
                 fTf(j,k,l) = conjg(eit)**2 * cmplx(re2, 0.0D0, dblprec)
               ! Smooth exterior complex scaling
               else  
                 fTf(j,k,l) = cmplx(re2, im2, dblprec)
               end if
               fTf(k,j,l) = fTf(j,k,l)          ! fTf is complex symmetric
            end do
         end do
      end do

      write (*,*) '        Error in numerical integration (analytic)', sqrt(numinterr),  &
     &          '(maximum), ', suminterr / real(nl * nstate * (nstate + 1) / 2,  &
     &          dblprec), '(mean)'
      deallocate(rework, imwork)
      end subroutine anader_fTfmtrx
