!
! oneel_Riss --- one-electron, fine-element matrix generator
! Mark Baertschy, Kevin Christ, Robin Santra, Christian Buth
! 24 Nov 2008
!
!
! This program gives the energy eigenvalues and radial eigenfunctions for a
! one-electron potential
!
! Version V1.0.0: 04 Dec 2008 (Namelist support added, write matrix elements
!       in the SES CAP format of oneel)
! Version V1.1.0: 23 Dec 2008 (Analytic dipole matrix elements; numerical
!       integration grid adjustment)
!

PROGRAM ATOM
 USE FiniteElements
  implicit none

!! This program gives the energy eigenvalues and radial eigenfunctions for a
!! one-electron potential

   INTEGER, ALLOCATABLE :: ndcnt(:), ndcntfin(:,:)
   INTEGER :: leftright(2), nodestot
   INTEGER :: nbasis, nstate, nl, ngrd, nvlnc
   INTEGER :: it0, it1, ict, ntot, i, j, k
   REAL(kind=8), ALLOCATABLE :: Eng(:,:), Cevec(:,:,:), Egfxn(:,:,:)
   REAL(kind=8), ALLOCATABLE :: Cevecfin(:,:,:)
   REAL(kind=8) :: rf, xh
   REAL(kind=8) :: c, eta
   TYPE(FEBnodes) :: xnd

      namelist/OneElectronRiss/ rf, nodestot, nvlnc, nl, nstate,  &
     &          c, eta, ngrd

      nodestot = 1001
      ngrd     = 3 * nodestot + 1
      read(*,nml=OneElectronRiss)

      ! Initialization
      write (*,*) 'One-electron with Riss-Meyer CAP V1.1.0, finite-element matrix generator'
      write (*,*) '(c) 2008 Mark Baertschy, Kevin Christ, Chris H. Greene, Robin Santra, Christian Buth'
      write (*,*)

!! LEFTRIGHT
!! leftright(2) enforces zero boundary conditions at either end of grid.
!! It is the optional last argument to CreateNodes.
!! If leftright argument is not specified then CreateNodes assumes a value
!!  of 2 for both elements of leftright.
!! This default value forces the function (but not its derivatives) to
!!  be zero at both ends of the grid (closed channels).

   leftright(1) = 2 ! include only f.e. types 1 and 2 (exclude type 0) on left
   leftright(2) = 2 ! include only f.e. types 1 and 2 (exclude type 0) on right

!! SQUARE ROOT MESH
!! Creates a square root mesh for the finite elements
!! The mesh starts at r=0
!! rf is the maximum value of the grid and the location of the last node
!! nodestot is the desired number of nodes
!! xnd is the output data structure containing information about
!!  finite element nodes
!! nbasis is the output number of basis functions

!   rf=60.0d0
!   nodestot=1001
   write (*,*) 'rf =', rf, ', nodestot =', nodestot
   CALL sqrtmesh( rf, nodestot, xnd, nbasis, leftright )


!!*********************************************************************!!

!! CALCULATE EIGENVECTORS AND EIGENVALUES FOR ALL l AND j

   CALL system_clock(it0, ict)

!! eigencalc calls an ARPACK routine to calculate eigenvalues and eigenvectors
!!  and calls aevmtrx, which creates the potential energy matrix

!! nvlnc is the number of the S shell of the valence electron

!! nl is the number of l quantum numbers starting with l=0
!! nstate is the total number (closed) eigenstates desired per l

!! ntot is the total number of eigenstates

!! Eigenvalues are returned in Eng, and the eigenvectors are returned in Cevec
!! ndcnt keeps track of the amount of wavefunction nodes in the ground state
!!  of each l

!   nvlnc = 5
!   nl = 3
!   nstate = 100
   write (*,*) 'nvlnc =', nvlnc, ', nl =', nl, ', nstate =', nstate

   ntot = nstate*nl
   ALLOCATE( Eng(nstate, nl), Cevec(nbasis, nstate, nl), ndcnt(nl) )
   CALL eigencalc(nbasis, nstate, nl, ntot, nvlnc, Eng, Cevec, ndcnt, xnd)
   CALL system_clock(it1)
   Print*,'ARPACK eigenvalue routine time is', ( (it1-it0)*1.d0/ict )

!!*********************************************************************!!

!! FUNCTION EVALUATION

!! Evaluate eigenfunctions on a grid
!! ngrd is the number of grid points fitted onto a grid of
!! size rf (from above)

!   ngrd=3*nodestot+1
   write (*,*) 'Numerical grid for eigenfunction evaluation ', ngrd
   ALLOCATE( Egfxn(ngrd, nstate, nl) )
   CALL eigenfxn( Cevec, ngrd, rf, xh, nbasis, nstate, nl, xnd, Egfxn )

!!*********************************************************************!!

   ALLOCATE( ndcntfin(nstate,nl) )

   DO k = 1, nl
     DO j = 1, nstate
       ndcntfin(j,k) = ndcnt(k) + (j-1)
     END DO
   END DO

!! MATRIX CREATION

!! Create matrix representations of the dipole and the CAP-W operators
!! in the atomic eigenvector basis
!! Energies and matrix elements are written to "atom.dat"

!   c = 4.0d0
!   eta = 0.1
   write (*,*) 'c =', c, ', eta =', eta

!! c defines the radius inside of which the complex absorbing potential
!! vanishes

   CALL mtrxbuild( Cevec, nbasis, nstate, nvlnc, ntot, nl, xnd, Eng, &
        ndcntfin, ngrd, Egfxn, xh, c, eta )

write(*,*)
write(*,*) 'Vive la reacción!'
END PROGRAM ATOM
