SUBROUTINE eigencalc(nbasis, nstate, nl, ntot, nvlnc, Eng, &
                     Cevec, ndcnt, xnd)

 USE FiniteElements

  implicit none


  INTEGER :: nbasis, nstate, nl, ntot, ndcnt(nl)
  INTEGER :: i, j, k, it0, it1, ndup
  INTEGER :: verbosity, nconv, nsmall, nconv2, nvlnc, nind, ebad
  REAL(kind=8) ::  Eng(nstate, nl), Cevec(nbasis, nstate, nl) 
  REAL(kind=8), ALLOCATABLE :: Evec(:,:), E(:) 
  REAL(kind=8), ALLOCATABLE :: V(:,:)
  REAL(kind=8), ALLOCATABLE :: O(:,:), KE(:,:), Ham(:,:)
  REAL(kind=8), ALLOCATABLE :: rr(:), rV(:), b(:), c(:), d(:)
  REAL(kind=8) :: ll
  REAL(kind=8) :: rf, r0, sigmar
  REAL(kind=8) :: evlnc
  TYPE(FEBnodes) :: xnd

!! nstate is the number of eigenvalues and eigenvectors desired
!! nl is the number of l quantum numbers taken into account starting with l=0
!! nvlnc is the number of the S shell of the valence electron

!! ll is the angular momentum quantum number l
!! nind is an index used to ensure nstate eigenvalues and eigenvectors are found
!!  for each l
!! ntot is the total number of eigenstates 
  
   nind = nstate + 3 + nvlnc
   ntot = nstate*nl
   ndcnt(:)=0.d0
 

   ALLOCATE( E(nind), Evec(nbasis,nind), V(-5:5,nbasis) )
   ALLOCATE( Ham(-5:5,nbasis), O(-5:5,nbasis), KE(-5:5,nbasis) )

!! Create kinetic energy matrix
  
   CALL KEmtrx(KE,xnd)

!! Create overlap matrix
  
   CALL Omtrx(O,xnd) 

   Print*,'Calculating eigenvectors and eigenvalues'


   ALLOCATE ( rr(441), rV(441), b(441), c(441), d(441) )

!! Read the formatted data file pot.dat containing r*V(r)
!! The atomic potential is calculated using, for example,
!! the program by Herman and Skillman

   open(unit=10,file='pot.dat',access='sequential',status='old',form='formatted')
   do j = 1, 441
      read(10,100) rr(j), rV(j)
100   format(e25.16,e25.16)
   end do
   close(unit=10)

   CALL spline(441, rr, rV, b, c, d)

   DO k = 1, nl  ! l loop begin

       ll = dble(k-1)
    
       CALL Vmtrx(V, ll, rr, rV, b, c, d, xnd)
   
!! Create complete Hamiltonian (in banded storage)
     
       Ham = KE + V
 
!! Calculate eigenvectors and eigenvalues

!! ARPACK routine for generalized, symmetric, positive-definite eigenproblem
!! Eigenvalues are returned in vector E, according to their proximity to sigmar
!! Eigenvectors are returned in columns of Evec
!! verbosity controls how much information is returned
!! All eigenvalues are calculated for l=0; after that, the valence
!!  shell electron's energy is set for sigmar
  

    IF (ll == 0) THEN
      nsmall = 3
      sigmar = -800.d0
    ELSE 
      nsmall = 0
      sigmar = evlnc
    END IF

    verbosity = 0
    nconv = 0
  
    IF ( nsmall > 0 ) THEN
    
       CALL eigenfew( Ham, O, nbasis, sigmar, E, Evec, nsmall, nconv, verbosity )
  
       sigmar = E(nconv)/2.d0
       
       IF (E(nconv) > 0.d0) THEN
          sigmar = 2*E(nconv)
          PRINT *, "This probably should not happen"
       END IF
       
    END IF
    
    CALL eigenfew( Ham, O, nbasis, sigmar, E(nconv+1), Evec(1,nconv+1), nind-nconv, & 
                   nconv2, verbosity )
    
    ndup = 0
     
    DO j = 1, nconv
       
       IF ( dabs( E(j)-E(nconv+1+ndup) ) < 1.0d-6 ) THEN
          ndup = ndup + 1
       END IF
       
    END DO
    
    IF ( ll == 0 ) THEN
       IF (nvlnc<=nconv) THEN 
          evlnc = E(nvlnc)
       ELSE
          evlnc = E(nvlnc+ndup)
       END IF
    END IF
    
    ebad = 0   
    DO j = 1, nind
       
       IF ( E(j) < evlnc ) THEN
          ebad = ebad + 1
       END IF
       
    END DO

!! Duplicate energies and those less than the ground state energy are discarded
!! Eigenvalues are stored in order in Eng
!! Eigenvectors are stored in order in Cevec
!! ndcnt is the amount of eigenfunction nodes in the ground state of each l number
    
    ndcnt(k)=0.d0
    IF (ll == 0) THEN
       CALL efndcnter( nbasis, Evec(1,1), ndcnt )    
       IF (ndcnt(k) /= 0) THEN
          Print*,'Ground state eigenfunction node error!'
       END IF
    END IF
     
    IF ( nvlnc>nconv ) THEN
       DO j = 1, nstate
          Eng(j,k) = E(ebad+j)
          Cevec(:,j,k) = Evec(:,ebad+j)      
       END DO
    ELSEIF ( nvlnc>(nconv-ndup) ) THEN
       DO j = 1, nstate
          Eng(j,k) = E(nvlnc+ndup+j-1)
          Cevec(:,j,k) = Evec(:, nvlnc+ndup+j-1)
       END DO
    ELSE
       DO j = 1 , nconv-ndup-ebad
          Eng(j,k) = E(ebad+j)
          Cevec(:,j,k) = Evec(:,ebad+j)
       END DO
       
       DO j = 1, nstate-(nconv-ndup-ebad)
          Eng(j+nconv-ndup-ebad,k) = E(nconv+j)
          Cevec(:,j+nconv-ndup-ebad,k) = Evec(:,nconv+j)
       END DO
    END IF
    
    CALL efndcnter(nbasis, Cevec(1,1,k), ndcnt(k))
    
    DO j = 1, nstate
       IF ( Cevec(1,j,k) < 0.d0 ) THEN
          Cevec(:,j,k) = -Cevec(:,j,k)
       END IF
    END DO
    
 END DO   ! l loop end    


 Print*,'Done'
 Print*,'The ground state energy is', evlnc
 Print*,'The number of l quantum numbers is', nl
 
 RETURN
END SUBROUTINE eigencalc
